
import Color
import Material
import SurfData
import TriangleMeshGeometry
import Vector2D
import Texture
import Appearance
import Shape

from pymol.OpenGL.GL import *
#from opengl.gl import *

#Semi-copy of Jogl's Decal class

TEXTURE_FILE = "texfilename"
VERTEX_NUM = "vertnum"
TEXTURE_X = "tx"
TEXTURE_Y = "ty"

class Decal():
	material = None
	shape = None
	def __init__(self, zipfile, decalNode, sd, images ):
		self.material = Material.Material()
		self.material.setColor(Color.Color(1.0,1.0,1.0)) # WHITE
		self.material.setSpecular(Color.Color(0.0,0.0,0.0)) #BLACK
		self.material.setAmbient(Color.Color(.7, .7, .7)) 
		
		
		mesh = self.buildDecalGeometry( decalNode, sd)
		appearance = self.buildAppearance(zipfile, decalNode.attrib[TEXTURE_FILE], images)
		self.shape = Shape.Shape(mesh, appearance)
		
	def buildDecalGeometry(self, decalNode, sd):
		tempTex = []
		vertexNums = []		
		vertexPos = []
		
		for i in range(0, len(sd.verts)):
			vertexPos.append(-1)			
		
		for textureNode in decalNode.getchildren():
			if (len(textureNode.attrib) > 0):
				vertNum = int(textureNode.attrib[VERTEX_NUM])
				tx = int(textureNode.attrib[TEXTURE_X])
				tx = tx / 1000.0
				ty = int(textureNode.attrib[TEXTURE_Y])
				ty = 1.0 - ty / 1000.0
				
				vertexPos[vertNum] = len(vertexNums)
				vertexNums.append(vertNum)
				
				tempTex.append(Vector2D.Vector2D(tx, ty))
		
		tempFaces = []
		for face in sd.faces:
			if (vertexPos[face[0]] > -1 and vertexPos[face[1]] > -1 and vertexPos[face[2]] > -1):
				tempFace = [None, None, None]
				tempFace[0] = vertexPos[face[0]]
				tempFace[1] = vertexPos[face[1]]
				tempFace[2] = vertexPos[face[2]]
				tempFaces.append(tempFace)
				
			
		decalVerts = []
		decalNormals = []
		decalFaces = []
		decalTextures = []
		decalColors = []
		
				
		for vertNum in vertexNums:
			decalVerts.append(sd.verts[vertNum])
			decalNormals.append(sd.normals[vertNum])
			decalColors.append([1.0, 1.0, 1.0, 0.0])
			
			
		#print "Decal vertex numbers: ",
		#print len(vertexNums)
		
		decalFaces = tempFaces
		decalTextures = tempTex
		decalMesh = TriangleMeshGeometry.TriangleMeshGeometry(decalVerts, decalNormals, decalColors, decalFaces, decalTextures)
		
		sd.numDecals += 1
		decalMesh.setPolygonOffset( -sd.numDecals)
		
		return decalMesh
			
			
	def buildAppearance(self, zipfile, textureFile, images):
		texFileName = textureFile
		if texFileName in images:
			#print "Image already loaded"
			return (images[texFileName])
		else:
			texture = Texture.Texture2D(texFileName, zipfile)
			texture.setMode(GL_MODULATE)
			
			appearance = Appearance.Appearance()
			appearance.setMaterial(self.material)
			appearance.setTexture(texture)
			
			
			images[texFileName] = appearance
			return appearance
		
	def Render(self):
		#This will render the mesh into the display list for the parent decal type
		self.shape.RenderNoList()