
#Material Class

import Color
from pymol.opengl.gl import *
#from opengl.gl import *
		
class Material:
	ambient = None #Assuming RBG Colors
	diffuse = None
	specular = None
	emission = None
	shininess = 64
	
	def __init__ (self, color = None):
		if (color == None):
			color = Color.Color(.5,.5,.5) # GRAY
		self.setColor(color)
		
	def setColor(self, color):
		self.ambient = Color.Color(color.getRed()/5, color.getGreen()/5, color.getBlue()/5)
		self.diffuse = color
		specular = Color.Color(.25,.25,.25) #LIGHT_GRAY
									
									
	def setAmbient(self, color):
		self.ambient = color
		
	def setDiffuse(self, color):
		self.diffuse = color
		
	def setSpecular(self, color):
		self.specular = color
		
	def setShininess(self, exponent):
		#Clamp
		if (exponent < 0):
			self.shininess = 0
		elif (exponent > 128):
			self.shininess = 128
		else:
			self.shininess = exponent
			
	def setEmission(self, color):
		self.emission = color
		
	def getAmbient(self):
		return self.ambient
		
	def getDiffuse(self):
		return self.diffuse
		
	def getSpecular(self):
		return self.specular
	
	def getShininess(self):
		return self.shininess
			
	def getEmission(self):
		return self.emission
		
	def Render(self):
		mat = [0.0, 0.0, 0.0, 1.0] #Alpha = 1
		if (self.ambient != None):
			mat[0] = self.ambient.getRed() / 255.0
			mat[1] = self.ambient.getGreen() / 255.0
			mat[2] = self.ambient.getBlue() / 255.0
			glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, mat)

		if ( self.diffuse != None ):
			mat[0] = self.diffuse.getRed()/255.0
			mat[1] = self.diffuse.getGreen()/255.0
			mat[2] = self.diffuse.getBlue()/255.0
			color = [ mat[0], mat[1], mat[2], 0.0 ] 
			glColor4fv( color ); 
    
		if (self.specular != None):
			mat[0] = self.specular.getRed() / 255.0;
			mat[1] = self.specular.getGreen() / 255.0;
			mat[2] = self.specular.getBlue() / 255.0;
			glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, mat );
    

		if ( self.emission != None ):
			mat[0] = self.emission.getRed() / 255.0;
			mat[1] = self.emission.getGreen() / 255.0;
			mat[2] = self.emission.getBlue() / 255.0;
			glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, mat);
    

		glMaterialf( GL_FRONT_AND_BACK, GL_SHININESS, self.shininess );
			
			