#Python Imports:
import struct
import os
import math
from xml.etree.ElementTree import * 

#Pymol Imports:


#Local imports:
import Vector3D
import SurfData
import Shape
import DecalList
import TriangleMeshGeometry
import Material
import Color
import Appearance


#Python idiom to mirror an enum

showSurfaceColor_none = 0
showSurfaceColor_bychain = 1
showSurfaceColor_charge = 2
showSurfaceColor_hydropathy = 3
	

chainColors = [[.54,  .70, 1.00], \
			[ .73,  .47, 1.00], \
			[1.00,  .72,  .30], \
			[1.00,  .29,  .42], \
			[ .42, 1.00,  .33], \
			[ .26,  .36,  .77], \
			[1.00,  .61,  .73], \
			[1.00,  .90,  .00]]
    
	
#private static strings
DECAL_LIST = "decallist"
SURFACE_FILE = "surfacefile"
FIELD_DENSE = "fielddense"
NAME = "name"
CHARGE = "charge"
AMBIENT_OCCLUSION = "ambient_occlusion"
HYDROPATHY = "hydropathy"
	

	

class Surface:
	#Class variables go here...
	chainNum = 0
	drawAbstracted = False
	numVertices = 0
	
	chargeValues = []
	ambientOcclusionValues = []
	hydropathyValues = []
	
	stickerTypeVis = []
	decalLists = []
	meshShape = None
	
	
	minX = float("+inf")
	minY = float("+inf")
	minZ = float("+inf")
	
	maxX = float("-inf")
	maxY = float("-inf")
	maxZ = float("-inf")
	

	
	def __init__(self, zipdata, surfaceNode, chainNum, drawAbstracted):
		
		self.chainNum = chainNum
		self.drawAbstracted = drawAbstracted
		#objfile = "some file name we need to get.ply"
		plyfilename = surfaceNode.attrib[self.getAbsPrefix() + SURFACE_FILE]
		
		#TODO - there's a better way of doing this...
		#Extract the file		
		plyfile = zipdata.extract(plyfilename)
		
		
		sd = SurfData.SurfData()				
		self.loadSurfacePLY(sd, plyfile)		
		
		#Then delete the file		
		os.remove(plyfile)
		
		self.numVertices = len(sd.verts)
		#print "numVertices: ", 
		#print self.numVertices
		
		#while (len(surfaceNode.getchildren()) > 0):
		for i in range(0, len(surfaceNode.getchildren())):
			child = surfaceNode.getchildren()[i]
			if drawAbstracted and (child.tag == DECAL_LIST) and (len(child.getchildren()) > 0):
				self.decalLists.append(DecalList.DecalList(zipdata, child, sd))				
			if (child.tag == FIELD_DENSE):
				#print "loading field data"
				self.addFieldDenseData(child)
				
		
		self.buildMesh(sd)
		for i in range (0, len(DecalList.DecalListType)):
			self.stickerTypeVis.append(True)
		self.stickerTypeVis[DecalList.DecalListType_sanded[0]] = False
		self.stickerTypeVis[DecalList.DecalListType_sanded_region[0]] = False
		
		#Kick in garbage collector here....
		sd = None
		
		

	def getAbsPrefix(self):
		if self.drawAbstracted:			
			return "abstract_" 
		else:
			return "orig_"
			
			
	def loadSurfacePLY(self, sd, inputfile):
		numBytesRead = 0
		
		#Open it in here? Sure, why not. Odds are this is more likely to be a stream?		
		#With beats a good try/finally block any day, according to python docs
		with open(inputfile, mode='rb') as plyfile :
			plyfile.seek(0,0)
			numVerts = 0
			hasMore = True
			while (hasMore):
				line = plyfile.readline()
				if (line != None) and (len(line) > 0):					
					#Tokenize the line.
					firstToken = None #Get the 1st token
					tokens = line.split() 		
					#print tokens
					if (len(tokens) > 0):
						firstToken = tokens[0]					
					if (firstToken == "element"):
						secondToken = None #Get the second Token
						if (len(tokens) > 1):
							secondToken = tokens[1]
						if (secondToken == "vertex"):
							thirdToken = None #Get the 3rd Token
							if (len(tokens) > 2):
								thirdToken = tokens[2]
							numVerts = int(thirdToken)							
							sd.verts = []
							for i in range(0, numVerts):
								sd.verts.append([])
					elif(firstToken == "end_header"):
						hasMore = False				
			numBytesRead = plyfile.tell()
						
			#Seek from the start to get to binary data
			plyfile.seek(numBytesRead, 0) 
			
			
			#The binary data itself is little-endian. 
			#The parser *could* detect that, but it appears that all of Greg's code assumes
			#little-endian and so shall I
			#Use "<f" for struct.unpack
			
			bindata = plyfile.read()
			offset = 0
			for i in range(0, numVerts):
				
				xvalue = bindata[offset*4: (offset+1)*4]
				offset += 1

				x = struct.unpack('<f', xvalue)[0]  #unpack string into float
				yvalue = bindata[offset*4: (offset+1)*4]
				offset += 1				
				y = struct.unpack('<f', yvalue)[0]				
				zvalue = bindata[offset*4: (offset+1)*4]
				offset += 1
				z = struct.unpack('<f', zvalue)[0]				
				sd.verts[i] = Vector3D.Vector3D(x,y,z)

			stripvalue = bindata[offset*4: (offset+1)*4]
			offset += 1
			numStripPts = struct.unpack('<i', stripvalue)[0]
			tstrips = []
		
			for i in range(0, numStripPts):
				rawval = bindata[offset*4: (offset+1)*4]
				offset += 1				
				intval = struct.unpack('<i', rawval)[0]
				#print intval
				tstrips.append(intval)
		
			# Convert triangle strips from -1 terminating
			# to length-preceeding (easier to unpack, later)
			length = 0
			# for (i = tstrips.length - 2, i >= 0, i--)
			for i in range(len(tstrips) - 2, -1, -1):
				if (tstrips[i] == -1):
					tstrips[i+1] = length
					length = 0
				else:
					tstrips[i+1] = tstrips[i]
					length = length + 1
			tstrips[0] = length
			
			
			#Unpack and figure out how many faces will be produces
			nfaces = 0
			i = 0
			while i < len(tstrips):
				nfaces = nfaces + tstrips[i] - 2
				i = i + tstrips[i] + 1
			
			#print nfaces
			#raw_input("")
			#init faces with empty values, so we don't loop out of bounds...
			sd.faces = []
			for i in range(0, nfaces):
				sd.faces.append([ None,None,None ]) #empty entry of 3 
				
			faceIndex = 0
			length = 0
			flip = False
			i = 0
			while (i < len(tstrips)):	
				
				self.maxX = max(self.maxX, sd.verts[tstrips[i]].x)
				self.maxY = max(self.maxY, sd.verts[tstrips[i]].y)
				self.maxZ = max(self.maxZ, sd.verts[tstrips[i]].z)
				
				self.minX = min(self.minX, sd.verts[tstrips[i]].x)				
				self.minY = min(self.minY, sd.verts[tstrips[i]].y)				
				self.minZ = min(self.minZ, sd.verts[tstrips[i]].z)
				
				if (length == 0):
					length = tstrips[i] - 2
					flip = False
					i += 2
				else:
					if (flip):
						sd.faces[faceIndex] = [tstrips[i-1], tstrips[i-2], tstrips[i]]
						
						faceIndex += 1
					else:
						sd.faces[faceIndex] = [tstrips[i-2], tstrips[i-1], tstrips[i]]
						faceIndex += 1
					flip = not flip
					length = length - 1
				
				i = i + 1 #loop increment		
			
		sd.calculateNorms()

	def buildMesh(self, sd):
		colors = self.calculateColor(showSurfaceColor_charge)
		meshGeometry = TriangleMeshGeometry.TriangleMeshGeometry(sd.verts, sd.normals, colors, sd.faces, None)
		
		meshGeometry.setPolygonOffset(1.0)
		
		material = Material.Material()
		material.setColor(Color.Color(1.0,1.0,1.0))
		material.setShininess(0)
		material.setSpecular(Color.Color(0.0, 0.0, 0.0))
		
		appearance = Appearance.Appearance()
		appearance.setMaterial(material)
		self.meshShape = Shape.Shape(meshGeometry, appearance)
		
	def disposeOfResources(self):
		#TODO - get the texturing stuff here working
		#How necessary is disposal with GC?
		for dl in self.decalLists:
			for i in range(0, dl.numChildren()):
				shape = dl.getChild(i)
				a = shape.getAppearance()
				a.getTexture().dispose()
		
		
	def getNumTextures(self):
		numTex = 0
		for dl in self.decalLists:
			numTex += dl.getNumUniqueImages()			
		return numTex
		
	def buildTextures(self):
		#TODO - figure out what this actually builds, if we need to...
		pass
		
		
	def setStickerTypeVisible(self, decaltype, val):		
		#Enable/disable the stickers available
		self.stickerTypeVis[decaltype[0]] = val
		
		
	def buildScene(self):
		pass
		
	def Render(self, visible):
		if (visible):			
			self.meshShape.Render()
		
		for i in range (0, len(self.decalLists)):
			listType = self.decalLists[i].listType
			if (listType == DecalList.DecalListType_unknown) or self.stickerTypeVis[listType[0]]:
				self.decalLists[i].Render()
		
		
		
	def expandBox(self, minvec, maxvec):
		maxvec.set(max(self.maxX, maxvec.x), \
				   max(self.maxY, maxvec.y), \
				   max(self.maxZ, maxvec.z))
		minvec.set(min(self.minX, minvec.x), \
			 	   min(self.minY, minvec.y), \
				   min(self.maxZ, maxvec.z))
		
		
	def setColor(self, coloring):
		colors = self.calculateColor(coloring)
		tmg = self.meshShape.getGeometry()
		tmg.SetColors(colors)		
		
		
		
	def addFieldDenseData(self, fieldDense):
		#This requires some tokenization of XML nodes, it appears. Suck		
		#Use string.split to tokenize on the ',' value
		#Use float(string) to convert back to float. Voila
		#print fieldDense.attrib[NAME]
		if True:
			if (fieldDense.attrib[NAME] == (self.getAbsPrefix() + CHARGE)):
				#parse out the Charge values
				valuestring = fieldDense.text
				tokens = valuestring.split(",")
				self.chargeValues = []
				for i in range(0, len(tokens)):
					self.chargeValues.append(float(tokens[i]))
				
				
			elif (fieldDense.attrib[NAME] == (self.getAbsPrefix() + AMBIENT_OCCLUSION)):
				#parse out the Occlusion values
				valuestring = fieldDense.text
				tokens = valuestring.split(",")
				self.ambientOcclusionValues = []
				for i in range(0, len(tokens)):
					self.ambientOcclusionValues.append(float(tokens[i]))
					
				
			elif (fieldDense.attrib[NAME] == (self.getAbsPrefix() + HYDROPATHY)):
				#parse out the Hydropathy values
				valuestring = fieldDense.text
				tokens = valuestring.split(",")
				self.hydropathyValues = []
				for i in range(0, len(tokens)):
					self.hydropathyValues.append(float(tokens[i]))
		
		
		
	#Coloring must be a member of showSurfaceColor....
	def calculateColor(self, coloring):
		#Evaluation of the booleans in order should make colorHydro false before
		#it gets to len(hydropathyValues), so it won't attempt a len(None)
		colorHydro = (coloring == showSurfaceColor_hydropathy)
		colorHydro = colorHydro and (self.hydropathyValues != None)
		colorHydro = colorHydro and (len(self.hydropathyValues) == self.numVertices)
		
		
		colorCharge = (coloring == showSurfaceColor_charge)
		colorCharge = colorCharge and (self.chargeValues != None)
		colorCharge = colorCharge and (len(self.chargeValues) == self.numVertices)	
		
		
		useAmbientOcc = self.ambientOcclusionValues != None 
		useAmbientOcc = useAmbientOcc and (len(self.ambientOcclusionValues) == self.numVertices)	
		
		colors = []
		for i in range(0, self.numVertices):
			colors.append([])
		
		#Force float math
		r = 1.0
		g = 1.0
		b = 1.0

		
		if (coloring == showSurfaceColor_bychain):
			chainPos = self.chainNum % 8
			r = chainColors[chainPos][0]  
			g = chainColors[chainPos][1] 
			b = chainColors[chainPos][2] 
					
		for i in range(0, self.numVertices):
			if (colorHydro == True):
				#TODO - this math needs looked at. Blues come out fine - reds do not.
				val = max(0.0, min(10, self.hydropathyValues[i]))
				if (val <= 5.0):
					r = val / 5.0
					g = .4 + val * (.6 / 5.0)
					b = .8 + val * (.2 / 5.0)
				else:
					r = 1.0 - (val - 5.0) / 5.0
					g = 1.0 - (val - 5.0) * (.2 / 5)
					b = 1 - (val - 5) * (.6 / 5.0)  
			
			elif (colorCharge == True):
				val = max(-13.0, min(13.0, self.chargeValues[i]))
				if (val < 0.0):
					r = 1.0
					g = (val + 13.0) / 13.0
					b = (val + 13.0) / 13.0
				else:
					r = 1.0 - (val / 13.0)
					g = 1.0 - (val / 13.0)
					b = 1.0
				
			if (useAmbientOcc == True):
				val = .9 * math.pow(self.ambientOcclusionValues[i], 1.5) + .1
				colors[i] = [r * val, g * val, b * val]
			else:
				colors[i] = [r, g, b]
				#print colors[i]
				
		return colors
		