

import Vector3D
import Vector2D

from pymol.OpenGL.GL import * 
#from OpenGL.GL import * 
import array

class TriangleMeshGeometry:
	vertices_buffer = []
	normals_buffer = []
	tex_coords_buffer = []
	color_buffer = []
	faces_buffer = []
	hasNormals = False
	hasTexture = False
	hasColors = False
	polyOffset = 0.0
	
	#GLDrawList info
	listbuilt = False	
	gllist = None
	
	#Vertex Buffer Objects
	vbobuilt = False
	vbo = None
		
	
	def __init__(self, vertices, normals, colors, faces, tex_coords):
		if (vertices == None):
			raise(Exception("vertices array null"))
			
		if (len(vertices) <= 0):
			raise(Exception("no rows in vertices array"))
		
		if (normals != None):
			if (len(normals) != len(vertices)):
				print "Normals count: " + len(normals)
				print "Vertices count: " + len(vertices)
				raise( Exception("normals rows != vertices rows"))
			else:
				self.hasNormals = True
		else:
			self.hasNormals = False
			
		if (colors != None):
			if (len(colors) != len(vertices)):
				print "Colors count: " + len(colors)
				print "Vertices count: " + len(vertices)
				raise(Exception("colors rows != vertices rows"))
			else:
				self.hasColors = True
		else:
			self.hasColors = False
			
		if (tex_coords != None):
			if (len(tex_coords) != len(vertices)):
				raise(Exception("tex coord rows != vertices rows"))
			else:
				self.hasTexture = True
		else:
			self.hasTexture = False
			
		coords = []
		vec_coords = []
		
		for i in range(0, len(vertices)):
			vec_coords = vertices[i].get()
			coords.append(vec_coords[0])
			coords.append(vec_coords[1])
			coords.append(vec_coords[2])
			
		
		self.vertices_buffer = coords
		
		index = 0
		coords = [] #get a new list
		if (self.hasNormals):
			for i in range(0, len(vertices)):
				vec_coords = normals[i].get()
				coords.append(vec_coords[0])
				coords.append(vec_coords[1])
				coords.append(vec_coords[2])
			self.normals_buffer = coords	
			
		if (self.hasColors):
			self.SetColors(colors)
			
		index = 0
		if (self.hasTexture):
			coords =[]
			point = []
			for i in range(0, len(vertices)):
				point = tex_coords[i]
				coords.append(point.x)
				coords.append(point.y)
			
			self.tex_coords_buffer = coords
			coords = []
				
		self.faces_buffer = []
		faceTemp = []
		
		#Verify this actually works and doesn't completely hose up on assigment.
		
		for i in range(0, len(faces)):
			for j in range(0,3):
				faceTemp.append(faces[i][j])
		
		self.faces_buffer = faceTemp
		
		#1 list for geometry every time....
		
	
			
	def SetColors(self, colors):
		numVerts = len(self.vertices_buffer) / 3
		coords = []
		for i in range(0, numVerts):
			#Colors should already be normalized in this list.
			coords.append(colors[i][0])
			coords.append(colors[i][1])
			coords.append(colors[i][2])
		self.color_buffer = coords
		coords = []
		
		self.listbuilt = False
		
		
		
	def ReverseNormals(self):
		normals = []
		for i in range(0, len(self.normals_buffer)):
			#This works because the normals_buffer is nothing but floats
			normals.append(-self.normals_buffer[i]) 
		self.normals_buffer = normals
		normals = []
	
	def setPolygonOffset(self, offset):
		self.polyOffset = offset
	
	
	def createList(self):
		if (self.gllist != None):
			glDeleteLists(self.gllist, 1)
		self.gllist = glGenLists(1)		
		glNewList(self.gllist, GL_COMPILE)
		self.RenderNoList()
		glEndList()
		self.listbuilt = True	

	def createVBO(self):
	self.vbobuilt
		if (self.vbo != None)
			glDeleteBuffers(1, vbo)
		
	#The Gotchas that may occur in this rendering part are the issues with 
	#PyOpenGL not playing nice with the EnableClientState for arrays
	def Render(self):		
		if not self.listbuilt:
			self.createList()
		if not self.vbobuilt:
			self.createVBO()
	
		#Start a DrawList...
		glCallList(self.gllist)
		
	
		
	def RenderNoList(self):		
	
		##### Render commands for list building
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
		glEnable(GL_CULL_FACE)
		glCullFace(GL_BACK)
		glEnable(GL_POLYGON_OFFSET_FILL)
		glPolygonOffset(1.0, self.polyOffset)
		
		if (self.hasTexture):
			glEnableClientState(GL_TEXTURE_COORD_ARRAY)
			glTexCoordPointer(2, GL_FLOAT, 8, self.tex_coords_buffer)
		
		if (self.hasColors):
			glEnableClientState(GL_COLOR_ARRAY)
			glColorPointer(3, GL_FLOAT, 12, self.color_buffer)			
		
		if (self.hasNormals):
			glEnableClientState(GL_NORMAL_ARRAY)
			glNormalPointer( GL_FLOAT, 12, self.normals_buffer)
			
		glEnableClientState(GL_VERTEX_ARRAY)
		glVertexPointer(3, GL_FLOAT, 0, self.vertices_buffer)


		glDrawElements(GL_TRIANGLES, len(self.faces_buffer), GL_UNSIGNED_INT, self.faces_buffer)	
		
		glDisableClientState(GL_VERTEX_ARRAY)
		if (self.hasTexture):
			glDisableClientState(GL_TEXTURE_COORD_ARRAY)
		if (self.hasColors):
			glDisableClientState(GL_COLOR_ARRAY)
		if (self.hasNormals):
			glDisableClientState(GL_NORMAL_ARRAY)
		
		glDisable(GL_BLEND)
		glDisable(GL_POLYGON_OFFSET_FILL)
	
	